(* Delphi Dialog
   Select files from a directory
   =============================
   
    J. Rathlev, D.24222 Schwentinental (kontakt(a)rathlev-home.de))

   The contents of this file may be used under the terms of the
   Mozilla Public License ("MPL") or
   GNU Lesser General Public License Version 2 or later (the "LGPL")

   Software distributed under this License is distributed on an "AS IS" basis,
   WITHOUT WARRANTY OF ANY KIND, either express or implied. See the License for
   the specific language governing rights and limitations under the License.
   
   Vers. 1 - February 2020
   last modified: Nov. 2021
   *)

unit SelectFilesDlg;

interface

uses Winapi.Windows, System.SysUtils, System.Classes, Vcl.Graphics, Vcl.Forms,
  Vcl.Controls, Vcl.ComCtrls, Vcl.StdCtrls, Vcl.Buttons, Vcl.ExtCtrls;

type
  TSelectFilesDialog = class(TForm)
    OKBtn: TBitBtn;
    CancelBtn: TBitBtn;
    lvFiles: TListView;
    laDir: TLabel;
    procedure FormCreate(Sender: TObject);
    procedure FormResize(Sender: TObject);
    procedure lvFilesColumnClick(Sender: TObject; Column: TListColumn);
    procedure lvFilesCompare(Sender: TObject; Item1, Item2: TListItem;
      Data: Integer; var Compare: Integer);
  private
    FCol: integer;
    FReverse: boolean;
{$IFDEF HDPI}   // scale glyphs and images for High DPI
    procedure AfterConstruction; override;
{$EndIf}
  public
    { Public declarations }
    function Execute (const Titel,Dir : string; var Files : string) : boolean;
  end;

function SelectFiles (const Titel,Dir : string; var Files : string) : boolean;

var
  SelectFilesDialog: TSelectFilesDialog;

{ ---------------------------------------------------------------- }
implementation

{$R *.DFM}

uses GnuGetText, FileUtils, StringUtils, System.DateUtils, System.Math, NumberUtils;

{ ---------------------------------------------------------------- }
procedure TSelectFilesDialog.FormCreate(Sender: TObject);
begin
  TranslateComponent (self,'dialogs');
  FCol:=0; FReverse:=false;
  end;

{$IFDEF HDPI}   // scale glyphs and images for High DPI
procedure TSelectFilesDialog.AfterConstruction;
begin
  inherited;
  if Application.Tag=0 then
    ScaleButtonGlyphs(self,PixelsPerInchOnDesign,Monitor.PixelsPerInch);
  end;
{$EndIf}

procedure TSelectFilesDialog.FormResize(Sender: TObject);
begin
  with lvFiles do begin
    Column[1].Width:=120;
    Column[2].Width:=70;
    Column[0].Width:=Width-215;
    end;
  end;

procedure TSelectFilesDialog.lvFilesColumnClick(Sender: TObject;
  Column: TListColumn);
begin
  if Column.Index=FCol then FReverse:=not FReverse
  else FReverse:=false;
  FCol:=Column.Index;
  lvFiles.AlphaSort;
  end;

procedure TSelectFilesDialog.lvFilesCompare(Sender: TObject; Item1,
  Item2: TListItem; Data: Integer; var Compare: Integer);
var
  dt1,dt2 : TDateTime;
  n1,n2   : int64;
begin
  if FCol=0 then Compare:=CompareText(Item1.Caption,Item2.Caption)
  else if (FCol=1) and (Item1.SubItems.Count>0) then begin  // Date
    if not TryStrToDateTime(Item1.SubItems[0],dt1) then dt1:=0;
    if not TryStrToDateTime(Item2.SubItems[0],dt2) then dt2:=0;
    Compare:=CompareDateTime(dt1,dt2);
    end
  else if (Item1.SubItems.Count>1) then begin // Size
    if not StrToSize(Item1.SubItems[1],n1) then n1:=0;
    if not StrToSize(Item2.SubItems[1],n2) then n2:=0;
    Compare:=CompareValue(n1,n2);
    end;
  if FReverse then Compare:=-Compare;
  end;

{ ---------------------------------------------------------------- }
function TSelectFilesDialog.Execute (const Titel,Dir: string; var Files : string) : boolean;
var
  FileInfo   : TSearchRec;
  Findresult : integer;
  i          : integer;
begin
  Caption:=Titel; laDir.Caption:=Dir;
  FindResult:=FindFirst(AddPath(Dir,'*.*'),faAllFiles,FileInfo);
  while FindResult=0 do with FileInfo do begin
    if NotSpecialDir(Name) then begin
      with lvFiles.Items.Add do begin
        Caption:=Name;
        SubItems.Add(DateTimeToStr(TimeStamp));
        SubItems.Add(SizeToStr(Size))
        end;
      end;
    FindResult:=FindNext (FileInfo);
    end;
  FindClose(FileInfo);
  Result:=ShowModal=mrOK;
  Files:='';
  if Result then with lvFiles.Items do for i:=0 to Count-1 do with Item[i] do begin
    if Selected then begin
      if length(Files)>0 then Files:=Files+Comma;
      Files:=Files+MakeQuotedStr(Caption,[Space]);
      end;
    end
  end;

function SelectFiles (const Titel,Dir : string; var Files : string) : boolean;
begin
  if not assigned(SelectFilesDialog) then begin
    SelectFilesDialog:=TSelectFilesDialog.Create(Application);
    end;
  Result:=SelectFilesDialog.Execute(Titel,Dir,Files);
  FreeAndNil(SelectFilesDialog);
  end;

end.
